{"changed":true,"filter":false,"title":"web-notification.js","tooltip":"/web-notification.js","value":"/*global module: false, define: false */\n\n/**\n * 'showNotification' callback.\n *\n * @callback ShowNotificationCallback\n * @param {error} [error] - The error object in case of any error\n * @param {function} [hide] - The hide notification function\n */\n\n/**\n * A simplified web notification API.\n *\n * @name webNotification\n * @namespace webNotification\n * @author Sagie Gur-Ari\n */\n\n/**\n * Initializes the web notification API.\n *\n * @function\n * @memberof! webNotification\n * @alias webNotification.initWebNotification\n * @private\n * @param {Object} global - The root context (window/global/...)\n * @param {function} factory - Returns a new instance of the API\n * @returns {Object} New instance of the API\n */\n(function initWebNotification(global, factory) {\n    'use strict';\n\n    /*istanbul ignore next*/\n    var NotificationAPI = global.Notification || window.Notification;\n\n    var webNotification = factory(NotificationAPI);\n\n    /**\n     * Initializes the web notification API (only used for testing).\n     *\n     * @function\n     * @memberof! webNotification\n     * @alias webNotification.initWebNotificationFromContext\n     * @private\n     * @param {Object} context - The root context (window/global/...)\n     * @returns {Object} New instance of the API\n     */\n    webNotification.initWebNotificationFromContext = function (context) {\n        return initWebNotification(context, factory);\n    };\n\n    if ((typeof define === 'function') && define.amd) {\n        define(function defineLib() {\n            return webNotification;\n        });\n    } else if ((typeof module === 'object') && module.exports) {\n        module.exports = webNotification;\n    } else {\n        global.webNotification = webNotification;\n    }\n\n    return webNotification;\n}(this, function initWebNotification(NotificationAPI) {\n    'use strict';\n\n    var webNotification = {};\n\n    /**\n     * The internal Notification library used by this library.\n     *\n     * @memberof! webNotification\n     * @alias webNotification.lib\n     * @private\n     */\n    webNotification.lib = NotificationAPI;\n\n    /**\n     * True to enable automatic requesting of permissions if needed.\n     *\n     * @member {Boolean}\n     * @memberof! webNotification\n     * @alias webNotification.allowRequest\n     * @public\n     */\n    webNotification.allowRequest = true; //true to enable automatic requesting of permissions if needed\n\n    /*eslint-disable func-name-matching*/\n    Object.defineProperty(webNotification, 'permissionGranted', {\n        /**\n         * Returns the permission granted value.\n         *\n         * @function\n         * @memberof! webNotification\n         * @private\n         * @returns {Boolean} True if permission is granted, else false\n         */\n        get: function getPermission() {\n            var permission = NotificationAPI.permission;\n\n            /**\n             * True if permission is granted, else false.\n             *\n             * @memberof! webNotification\n             * @alias webNotification.permissionGranted\n             * @public\n             */\n            var permissionGranted = false;\n            if (permission === 'granted') {\n                permissionGranted = true;\n            }\n\n            return permissionGranted;\n        }\n    });\n    /*eslint-enable func-name-matching*/\n\n    /**\n     * Empty function.\n     *\n     * @function\n     * @memberof! webNotification\n     * @alias webNotification.noop\n     * @private\n     * @returns {undefined} Undefined\n     */\n    var noop = function () {\n        return undefined;\n    };\n\n    /**\n     * Checks if web notifications are permitted.\n     *\n     * @function\n     * @memberof! webNotification\n     * @alias webNotification.isEnabled\n     * @private\n     * @returns {Boolean} True if allowed to show web notifications\n     */\n    var isEnabled = function () {\n        return webNotification.permissionGranted;\n    };\n\n    /**\n     * Displays the web notification and returning a 'hide' notification function.\n     *\n     * @function\n     * @memberof! webNotification\n     * @alias webNotification.createAndDisplayNotification\n     * @private\n     * @param {String} title - The notification title text (defaulted to empty string if null is provided)\n     * @param {Object} options - Holds the notification data (web notification API spec for more info)\n     * @param {String} [options.icon=/favicon.ico] - The notification icon (defaults to the website favicon.ico)\n     * @param {Number} [options.autoClose] - Auto closes the notification after the provided amount of millies (0 or undefined for no auto close)\n     * @param {function} [options.onClick] - An optional onclick event handler\n     * @returns {function} The hide notification function\n     */\n    var createAndDisplayNotification = function (title, options) {\n        var autoClose = 0;\n        if (options.autoClose && (typeof options.autoClose === 'number')) {\n            autoClose = options.autoClose;\n        }\n\n        //defaults the notification icon to the website favicon.ico\n        if (!options.icon) {\n            options.icon = '/favicon.ico';\n        }\n\n        var notification = new NotificationAPI(title, options);\n\n        //add onclick handler\n        if (options.onClick && notification) {\n            notification.onclick = options.onClick;\n        }\n\n        var hideNotification = function () {\n            notification.close();\n        };\n\n        if (autoClose) {\n            setTimeout(hideNotification, autoClose);\n        }\n\n        return hideNotification;\n    };\n\n    /**\n     * Returns an object with the show notification input.\n     *\n     * @function\n     * @memberof! webNotification\n     * @alias webNotification.parseInput\n     * @private\n     * @param {Array} argumentsArray - An array of all arguments provided to the show notification function\n     * @returns {Object} The parsed data\n     */\n    var parseInput = function (argumentsArray) {\n        //callback is always the last argument\n        var callback = noop;\n        if (argumentsArray.length && (typeof argumentsArray[argumentsArray.length - 1] === 'function')) {\n            callback = argumentsArray.pop();\n        }\n\n        var title = null;\n        var options = null;\n        if (argumentsArray.length === 2) {\n            title = argumentsArray[0];\n            options = argumentsArray[1];\n        } else if (argumentsArray.length === 1) {\n            var value = argumentsArray.pop();\n            if (typeof value === 'string') {\n                title = value;\n                options = {};\n            } else {\n                title = '';\n                options = value;\n            }\n        }\n\n        //set defaults\n        title = title || '';\n        options = options || {};\n\n        return {\n            callback: callback,\n            title: title,\n            options: options\n        };\n    };\n\n    /**\n     * Shows the notification based on the provided input.<br>\n     * The callback invoked will get an error object (in case of an error, null in\n     * case of no errors) and a 'hide' function which can be used to hide the notification.\n     *\n     * @function\n     * @memberof! webNotification\n     * @alias webNotification.showNotification\n     * @public\n     * @param {String} [title] - The notification title text (defaulted to empty string if null is provided)\n     * @param {Object} [options] - Holds the notification data (web notification API spec for more info)\n     * @param {String} [options.icon=/favicon.ico] - The notification icon (defaults to the website favicon.ico)\n     * @param {Number} [options.autoClose] - Auto closes the notification after the provided amount of millies (0 or undefined for no auto close)\n     * @param {function} [options.onClick] - An optional onclick event handler\n     * @param {ShowNotificationCallback} [callback] - Called after the show is handled.\n     * @example\n     * ```js\n     * $('.some-button').on('click', function onClick() {\n     *   webNotification.showNotification('Example Notification', {\n     *     body: 'Notification Text...',\n     *     icon: 'my-icon.ico',\n     *     onClick: function onNotificationClicked() {\n     *       console.log('Notification clicked.');\n     *     },\n     *     autoClose: 4000 //auto close the notification after 4 seconds (you can manually close it via hide function)\n     *   }, function onShow(error, hide) {\n     *     if (error) {\n     *       window.alert('Unable to show notification: ' + error.message);\n     *     } else {\n     *       console.log('Notification Shown.');\n     *\n     *       setTimeout(function hideNotification() {\n     *         console.log('Hiding notification....');\n     *         hide(); //manually close the notification (you can skip this if you use the autoClose option)\n     *       }, 5000);\n     *     }\n     *   });\n     * });\n     * ```\n     */\n    webNotification.showNotification = function () {\n        //convert to array to enable modifications\n        var argumentsArray = Array.prototype.slice.call(arguments, 0);\n\n        if ((argumentsArray.length >= 1) && (argumentsArray.length <= 3)) {\n            var data = parseInput(argumentsArray);\n\n            //get values\n            var callback = data.callback;\n            var title = data.title;\n            var options = data.options;\n\n            var hideNotification = null;\n            if (isEnabled()) {\n                hideNotification = createAndDisplayNotification(title, options);\n                callback(null, hideNotification);\n            } else if (webNotification.allowRequest) {\n                NotificationAPI.requestPermission(function onRequestDone() {\n                    if (isEnabled()) {\n                        hideNotification = createAndDisplayNotification(title, options);\n                        callback(null, hideNotification);\n                    } else {\n                        callback(new Error('Notifications are not enabled.'), null);\n                    }\n                });\n            } else {\n                callback(new Error('Notifications are not enabled.'), null);\n            }\n        }\n    };\n\n    return webNotification;\n}));\n","undoManager":{"mark":99,"position":100,"stack":[[{"start":{"row":32,"column":8},"end":{"row":32,"column":9},"action":"insert","lines":["c"],"id":244}],[{"start":{"row":32,"column":9},"end":{"row":32,"column":10},"action":"insert","lines":["o"],"id":245}],[{"start":{"row":32,"column":10},"end":{"row":32,"column":11},"action":"insert","lines":["n"],"id":246}],[{"start":{"row":32,"column":11},"end":{"row":32,"column":12},"action":"insert","lines":["t"],"id":247}],[{"start":{"row":32,"column":12},"end":{"row":32,"column":13},"action":"insert","lines":["e"],"id":248}],[{"start":{"row":32,"column":13},"end":{"row":32,"column":14},"action":"insert","lines":["x"],"id":249}],[{"start":{"row":32,"column":14},"end":{"row":32,"column":15},"action":"insert","lines":["t"],"id":250}],[{"start":{"row":32,"column":14},"end":{"row":32,"column":15},"action":"remove","lines":["t"],"id":251}],[{"start":{"row":32,"column":13},"end":{"row":32,"column":14},"action":"remove","lines":["x"],"id":252}],[{"start":{"row":32,"column":12},"end":{"row":32,"column":13},"action":"remove","lines":["e"],"id":253}],[{"start":{"row":32,"column":11},"end":{"row":32,"column":12},"action":"remove","lines":["t"],"id":254}],[{"start":{"row":32,"column":10},"end":{"row":32,"column":11},"action":"remove","lines":["n"],"id":255}],[{"start":{"row":32,"column":9},"end":{"row":32,"column":10},"action":"remove","lines":["o"],"id":256}],[{"start":{"row":32,"column":8},"end":{"row":32,"column":9},"action":"remove","lines":["c"],"id":257}],[{"start":{"row":32,"column":8},"end":{"row":32,"column":9},"action":"insert","lines":["t"],"id":258}],[{"start":{"row":32,"column":9},"end":{"row":32,"column":10},"action":"insert","lines":["y"],"id":259}],[{"start":{"row":32,"column":10},"end":{"row":32,"column":11},"action":"insert","lines":["p"],"id":260}],[{"start":{"row":32,"column":11},"end":{"row":32,"column":12},"action":"insert","lines":["e"],"id":261}],[{"start":{"row":32,"column":12},"end":{"row":32,"column":13},"action":"insert","lines":["s"],"id":262}],[{"start":{"row":32,"column":13},"end":{"row":32,"column":14},"action":"insert","lines":[" "],"id":263}],[{"start":{"row":32,"column":14},"end":{"row":32,"column":15},"action":"insert","lines":["="],"id":264}],[{"start":{"row":32,"column":15},"end":{"row":32,"column":16},"action":"insert","lines":[" "],"id":265}],[{"start":{"row":32,"column":16},"end":{"row":32,"column":18},"action":"insert","lines":["[]"],"id":266}],[{"start":{"row":32,"column":17},"end":{"row":32,"column":18},"action":"insert","lines":["g"],"id":267}],[{"start":{"row":32,"column":18},"end":{"row":32,"column":19},"action":"insert","lines":["l"],"id":268}],[{"start":{"row":32,"column":19},"end":{"row":32,"column":20},"action":"insert","lines":["o"],"id":269}],[{"start":{"row":32,"column":20},"end":{"row":32,"column":21},"action":"insert","lines":["b"],"id":270}],[{"start":{"row":32,"column":21},"end":{"row":32,"column":22},"action":"insert","lines":["a"],"id":271}],[{"start":{"row":32,"column":22},"end":{"row":32,"column":23},"action":"insert","lines":["l"],"id":272}],[{"start":{"row":32,"column":23},"end":{"row":32,"column":24},"action":"insert","lines":[","],"id":273}],[{"start":{"row":32,"column":24},"end":{"row":32,"column":25},"action":"insert","lines":[" "],"id":274}],[{"start":{"row":32,"column":25},"end":{"row":32,"column":26},"action":"insert","lines":["w"],"id":275}],[{"start":{"row":32,"column":26},"end":{"row":32,"column":27},"action":"insert","lines":["i"],"id":276}],[{"start":{"row":32,"column":27},"end":{"row":32,"column":28},"action":"insert","lines":["n"],"id":277}],[{"start":{"row":32,"column":28},"end":{"row":32,"column":29},"action":"insert","lines":["d"],"id":278}],[{"start":{"row":32,"column":29},"end":{"row":32,"column":30},"action":"insert","lines":["o"],"id":279}],[{"start":{"row":32,"column":30},"end":{"row":32,"column":31},"action":"insert","lines":["w"],"id":280}],[{"start":{"row":32,"column":32},"end":{"row":32,"column":33},"action":"insert","lines":[";"],"id":281}],[{"start":{"row":32,"column":33},"end":{"row":33,"column":0},"action":"insert","lines":["",""],"id":282},{"start":{"row":33,"column":0},"end":{"row":33,"column":4},"action":"insert","lines":["    "]}],[{"start":{"row":33,"column":4},"end":{"row":33,"column":5},"action":"insert","lines":["v"],"id":283}],[{"start":{"row":33,"column":5},"end":{"row":33,"column":6},"action":"insert","lines":["a"],"id":284}],[{"start":{"row":33,"column":6},"end":{"row":33,"column":7},"action":"insert","lines":["r"],"id":285}],[{"start":{"row":33,"column":7},"end":{"row":33,"column":8},"action":"insert","lines":[" "],"id":286}],[{"start":{"row":33,"column":8},"end":{"row":33,"column":9},"action":"insert","lines":["i"],"id":287}],[{"start":{"row":33,"column":9},"end":{"row":33,"column":10},"action":"insert","lines":["n"],"id":288}],[{"start":{"row":33,"column":10},"end":{"row":33,"column":11},"action":"insert","lines":["d"],"id":289}],[{"start":{"row":33,"column":11},"end":{"row":33,"column":12},"action":"insert","lines":["e"],"id":290}],[{"start":{"row":33,"column":12},"end":{"row":33,"column":13},"action":"insert","lines":["x"],"id":291}],[{"start":{"row":33,"column":13},"end":{"row":33,"column":14},"action":"insert","lines":[";"],"id":292}],[{"start":{"row":33,"column":14},"end":{"row":34,"column":0},"action":"insert","lines":["",""],"id":293},{"start":{"row":34,"column":0},"end":{"row":34,"column":4},"action":"insert","lines":["    "]}],[{"start":{"row":34,"column":4},"end":{"row":34,"column":5},"action":"insert","lines":["f"],"id":294}],[{"start":{"row":34,"column":5},"end":{"row":34,"column":6},"action":"insert","lines":["o"],"id":295}],[{"start":{"row":34,"column":6},"end":{"row":34,"column":7},"action":"insert","lines":["r"],"id":296}],[{"start":{"row":34,"column":7},"end":{"row":34,"column":8},"action":"insert","lines":[" "],"id":297}],[{"start":{"row":34,"column":8},"end":{"row":34,"column":10},"action":"insert","lines":["()"],"id":298}],[{"start":{"row":34,"column":9},"end":{"row":34,"column":10},"action":"insert","lines":["i"],"id":299}],[{"start":{"row":34,"column":10},"end":{"row":34,"column":11},"action":"insert","lines":["n"],"id":300}],[{"start":{"row":34,"column":11},"end":{"row":34,"column":12},"action":"insert","lines":["d"],"id":301}],[{"start":{"row":34,"column":12},"end":{"row":34,"column":13},"action":"insert","lines":["e"],"id":302}],[{"start":{"row":34,"column":13},"end":{"row":34,"column":14},"action":"insert","lines":["x"],"id":303}],[{"start":{"row":34,"column":14},"end":{"row":34,"column":15},"action":"insert","lines":[" "],"id":304}],[{"start":{"row":34,"column":15},"end":{"row":34,"column":16},"action":"insert","lines":["="],"id":305}],[{"start":{"row":34,"column":16},"end":{"row":34,"column":17},"action":"insert","lines":[" "],"id":306}],[{"start":{"row":34,"column":17},"end":{"row":34,"column":18},"action":"insert","lines":["0"],"id":307}],[{"start":{"row":34,"column":18},"end":{"row":34,"column":19},"action":"insert","lines":[";"],"id":308}],[{"start":{"row":34,"column":19},"end":{"row":34,"column":20},"action":"insert","lines":[" "],"id":309}],[{"start":{"row":34,"column":20},"end":{"row":34,"column":21},"action":"insert","lines":["i"],"id":310}],[{"start":{"row":34,"column":21},"end":{"row":34,"column":22},"action":"insert","lines":["n"],"id":311}],[{"start":{"row":31,"column":4},"end":{"row":34,"column":23},"action":"remove","lines":["","    var types = [global, window];","    var index;","    for (index = 0; in)"],"id":404}],[{"start":{"row":31,"column":4},"end":{"row":32,"column":0},"action":"remove","lines":["",""],"id":405}],[{"start":{"row":31,"column":0},"end":{"row":31,"column":4},"action":"remove","lines":["    "],"id":406}],[{"start":{"row":31,"column":0},"end":{"row":32,"column":0},"action":"insert","lines":["",""],"id":407}],[{"start":{"row":32,"column":0},"end":{"row":32,"column":1},"action":"insert","lines":[" "],"id":408}],[{"start":{"row":32,"column":1},"end":{"row":32,"column":2},"action":"insert","lines":[" "],"id":409}],[{"start":{"row":32,"column":2},"end":{"row":32,"column":3},"action":"insert","lines":[" "],"id":410}],[{"start":{"row":32,"column":3},"end":{"row":32,"column":4},"action":"insert","lines":[" "],"id":411}],[{"start":{"row":32,"column":4},"end":{"row":32,"column":5},"action":"insert","lines":["/"],"id":412}],[{"start":{"row":32,"column":5},"end":{"row":32,"column":6},"action":"insert","lines":["*"],"id":413}],[{"start":{"row":32,"column":6},"end":{"row":32,"column":7},"action":"insert","lines":["*"],"id":414}],[{"start":{"row":32,"column":7},"end":{"row":32,"column":8},"action":"insert","lines":["/"],"id":415}],[{"start":{"row":32,"column":6},"end":{"row":32,"column":7},"action":"insert","lines":["i"],"id":416}],[{"start":{"row":32,"column":7},"end":{"row":32,"column":8},"action":"insert","lines":["s"],"id":417}],[{"start":{"row":32,"column":8},"end":{"row":32,"column":9},"action":"insert","lines":["t"],"id":418}],[{"start":{"row":32,"column":9},"end":{"row":32,"column":10},"action":"insert","lines":["a"],"id":419}],[{"start":{"row":32,"column":10},"end":{"row":32,"column":11},"action":"insert","lines":["n"],"id":420}],[{"start":{"row":32,"column":11},"end":{"row":32,"column":12},"action":"insert","lines":["b"],"id":421}],[{"start":{"row":32,"column":12},"end":{"row":32,"column":13},"action":"insert","lines":["u"],"id":422}],[{"start":{"row":32,"column":13},"end":{"row":32,"column":14},"action":"insert","lines":["l"],"id":423}],[{"start":{"row":32,"column":14},"end":{"row":32,"column":15},"action":"insert","lines":[" "],"id":424}],[{"start":{"row":32,"column":15},"end":{"row":32,"column":16},"action":"insert","lines":["i"],"id":425}],[{"start":{"row":32,"column":16},"end":{"row":32,"column":17},"action":"insert","lines":["g"],"id":426}],[{"start":{"row":32,"column":17},"end":{"row":32,"column":18},"action":"insert","lines":["n"],"id":427}],[{"start":{"row":32,"column":18},"end":{"row":32,"column":19},"action":"insert","lines":["o"],"id":428}],[{"start":{"row":32,"column":19},"end":{"row":32,"column":20},"action":"insert","lines":["r"],"id":429}],[{"start":{"row":32,"column":20},"end":{"row":32,"column":21},"action":"insert","lines":["e"],"id":430}],[{"start":{"row":32,"column":21},"end":{"row":32,"column":22},"action":"insert","lines":[" "],"id":431}],[{"start":{"row":32,"column":22},"end":{"row":32,"column":23},"action":"insert","lines":["n"],"id":432}],[{"start":{"row":32,"column":23},"end":{"row":32,"column":24},"action":"insert","lines":["e"],"id":433}],[{"start":{"row":32,"column":24},"end":{"row":32,"column":25},"action":"insert","lines":["x"],"id":434}],[{"start":{"row":32,"column":25},"end":{"row":32,"column":26},"action":"insert","lines":["t"],"id":435}],[{"start":{"row":44,"column":23},"end":{"row":44,"column":29},"action":"remove","lines":["global"],"id":436},{"start":{"row":44,"column":23},"end":{"row":44,"column":30},"action":"insert","lines":["context"]}]]},"ace":{"folds":[],"scrolltop":442.5,"scrollleft":0,"selection":{"start":{"row":44,"column":30},"end":{"row":44,"column":30},"isBackwards":false},"options":{"guessTabSize":true,"useWrapMode":false,"wrapToView":true},"firstLineState":{"row":30,"state":"start","mode":"ace/mode/javascript"}},"timestamp":1489076878264}